"""Connector objects."""
from circadapt.components import Component


class Connector(Component):
    """Connector objects."""

    parameters = [
        ]

    signals = [
        'q',
        ]


class ArtVen(Component):
    """
    ArtVen connects two nodes with a passive, flow-dependent resistance.

    Parameters
    ==========
    k: double
        Exponent in pressure-flow relationship
    p0: double
        Reference pressure at q=q0
    q0: double
        Reference flow at p=p0

    Signals
    ==========
    q: float
        Flow over ArtVen
    """

    parameters = [
        'p0',
        'q0',
        'k',
        ]

    signals = [
        'q',
        ]


class ArtVen2022(Component):
    """ArtVen2022 object."""

    parameters = [
        'p0AV',
        'q0AV',
        'kAV',
        ]

    signals = [
        'q',
        ]


class Diode(Connector):
    """
    Ideal diode describing flow between nodes at positive pressure gradient.


    Parameters
    ==========
    R: double
        Resistance

    Signals
    ==========
    q: float
        Flow through the valve
    """
    parameters = [
        'R',
        ]

    signals = [
        'q',
        ]


class Resistance(Connector):
    """
    Simple Resistance describing flow between nodes.


    Parameters
    ==========
    R: double
        Resistance

    Signals
    ==========
    q: float
        Flow through the valve
    """

    parameters = [
        'R',
        ]

    signals = [
        'q',
        ]


class Valve2022(Component):
    r"""
    Valve2022 valve connects two nodes.

    Parameters
    ==========
    adaptation_Aopen_fac: double
        factor used
    A_open: float
        Opening area
    A_leak: float
        Leaking valve area
    l: float
        Length of valve
    rho_b: float
        Blood density
    papillary_muscles: bool
        If true, papilary muscle implementation is activated
    soft_closure: bool
        If true, a soft closure is activated

    Signals
    ==========
    q: float
        Flow through the valve
    qDot: float
        Time derivative of the flow q

    """

    parameters = [
        'adaptation_A_open_fac',
        'A_open',
        'A_leak',
        'l',
        'rho_b',
        'papillary_muscles',
        'papillary_muscles_slope',
        'papillary_muscles_min',
        'papillary_muscles_A_open_fac',
        'soft_closure',
        ]

    signals = [
        'q',
        'qDot',
        'L',
        'A',
        'Dp',
        'DpB',
        'R',
        'diast',
        ]
