import numpy as np

from circadapt.model.walmsley2015 import _set_general, _set_ArtVen, \
    _set_TriSeg, _set_cavity_volumes, _set_wall, _set_bag, _set_valve, \
    _get_general, _get_log, _get_artven, _get_chamber, _get_triseg, \
    _get_cavity, _get_wall, _get_node, _get_bag, _get_valve, \
    _set_component_parameters

    
def set_model_state(CA, Pdict):
    _set_general(CA, Pdict)
    _set_ArtVen(CA, Pdict)
    # Chamber: nothing
    _set_TriSeg(CA, Pdict)
    _set_cavity_volumes(CA, Pdict)
    _set_wall(CA, Pdict)
    _set_patch(CA, Pdict)
    # Node: nothing
    _set_bag(CA, Pdict)
    _set_valve(CA, Pdict)
    # Adapt: TODO: implement adapt

def _set_patch(CA, P):
    par = {'dT','LsRef','vMax','TimeAct','VWall','AmRef',
           'kP','EfP0','SfA','SfP','hSe',
           'fXbA','fXbD','fTnA','fTnD'}

    par_ORD = {'duration', 'YCa', 'CaRef', 'TnConc','FacGNa','FacGNaL','FacGto',
               'FacGncx','Facpca','Facpcab','FacJrel','FacJup','Faccsqnmax',
               'Factff','Factfs','Factfcaf','Factfcas'}

    state_variables = { 'Lsi' }
    state_variables_ORD = {  'v','nai','nass',
                        'ki','kss','cai','cass','cansr',
                        'cajsr','m','hf','hs','j','hsp',
                        'jp','mL','hL','hLp','a','iF',
                        'iS','ap','iFp','iSp','d','ff','fs','fcaf',
                        'fcas','jca','nca','ffp','fcafp','xrf','xrs',
                        'xs1','xs2','xk1','Jrelnp','Jrelp','CaMKt'}

    state_variable_vectors = [['Xbx','nx'],['Tnx','nx']]

    for iL in range(len(P['Patch']['Name'])):
        # get location name
        if P['Patch']['Name'][iL][1]=='a':
            # atria
            wall = P['Patch']['Name'][iL][0:2]
            patch = P['Patch']['Name'][iL]
            loc_name = 'Model.Peri.'+wall+'.W.p'+patch+'.'
        elif P['Patch']['Name'][iL][1]=='v':
            # ventricles
            wall = P['Patch']['Name'][iL][0:2]
            patch = P['Patch']['Name'][iL]
            loc_name = 'Model.Peri.TriSeg.w'+wall+'.p'+patch+'.'
        else:
            raise ValueError('Patch name not recognized')

        # set parameters
        _set_component_parameters(
            CA, par, P['Patch'], loc_name, iL)

        # set parameters ORD
        _set_component_parameters(
            CA, par_ORD, P['Patch'], loc_name+'ORD.', iL)

        # set state variables
        _set_component_parameters(
            CA, state_variables, P['Patch'], loc_name, iL)

        # set state variables ORD
        _set_component_parameters(
            CA, state_variables_ORD, P['Patch'], loc_name+'ORD.', iL)

        # set state variable vectors Xbx and Tnx
        # TODO: allow change in nx
        for p in state_variable_vectors:
            if p[0] in P['Patch'].keys():
                v = P['Patch'][p[0]][-1, iL, :]
                for j in range(len(v)):
                    CA.set(loc_name+p[0]+':'+str(j+1),v[j], float)
            else:
                print('key ' + p + ' not found')


####################################
# Get Pdict
####################################
def get_model_state(CA):
    Pdict = {}

    Pdict['t'] = CA._get_double_vector('Solver.Time')
    Pdict['General'] = _get_general(CA)
    Pdict['Log'] = _get_log(CA)
    Pdict['ArtVen'] = _get_artven(CA)
    Pdict['Chamber'] = _get_chamber(CA)
    Pdict['TriSeg'] = _get_triseg(CA)
    Pdict['Cavity'] =_get_cavity(CA)
    Pdict['Wall'] = _get_wall(CA)
    Pdict['Patch'] = _get_patch(CA, Pdict)
    Pdict['Node'] = _get_node(CA)
    Pdict['Bag'] = _get_bag(CA)
    Pdict['Valve'] = _get_valve(CA)
    return Pdict

def _get_log(CA):
    log = {}
        # Log
    log['Code']='cpp'
    log['Version']=CA._model.getVersion(CA._idx_model)
    log['WrapperLanguage']='Python'
    log['WrapperVersion']=CA.version
    return log

def _get_patch(CA, P):
    patch = {}

    wallname = P['Wall']['Name']
    nPatch = int(sum(P['Wall']['nPatch']))
    patchnames = np.zeros((nPatch,), dtype=object)
    iPatchName = 0
    for iWall in range(len(wallname)):
        for iPatch in range(P['Wall']['nPatch'][iWall]):
            patchnames[P['Wall']['iPatch'][iWall]-1+iPatch]=wallname[iWall] + str(iPatch+1)

    patch_loc_cpp = np.array( patchnames)
    for i in range(len(patchnames)):
        patch_loc_cpp[i] = 'Model.Peri.'
        if patchnames[i][1] == 'a':
            patch_loc_cpp[i] = patch_loc_cpp[i] + patchnames[i][:2] + '.W.p'+patchnames[i]+'.'
        elif patchnames[i][1] == 'v':
            patch_loc_cpp[i] = patch_loc_cpp[i] + 'TriSeg.w' + patchnames[i][:2] + '.p'+patchnames[i]+'.'

    patch['Name'] = patchnames
    patch['n'] = nPatch
        
    par = {'dT','LsRef','vMax','TimeAct','VWall','AmRef',
                                  'kP','EfP0','SfA','SfP','hSe','fXbA','fXbD','fTnA','fTnD'}
    par_ord = {'YCa','CaRef', 'duration','TnConc','FacGNa','FacGNaL','FacGto',
               'FacGncx','Facpca','Facpcab','FacJrel','FacJup','Faccsqnmax',
               'Factff','Factfs','Factfcaf','Factfcas'} 
    
    parvec = ['Lsi','T','Ef','Ls','SfEcm','SfPasT','LsiDot','Sf','DSfDEf',
              'DADT','Am0','Am','TnDot']
                               #'Xb','XbDot', ]
    parvec_ord= { 'Ca', 'v','nai','nass',
                               'ki','kss','cai','cass','cansr',
                               'cajsr','m','hf','hs','j','hsp',
                               'jp','mL','hL','hLp','a','iF',
                               'iS','ap','iFp','iSp','d','ff','fs','fcaf',
                               'fcas','jca','nca','ffp','fcafp','xrf','xrs',
                               'xs1','xs2','xk1','Jrelnp','Jrelp','CaMKt'}
                               #'qCaCell','qCaSarc'}
    parmat = [['Xbx','nx'],['Tnx','nx']]#,['TnxDot','nx'],['Overlap','nx'],['ExpSx','nx']]
    
    for p in par:
        patch[p] = np.ndarray(nPatch)
        patch[p][:]=0
        for iP in range(len(patchnames)):
            patch[p][iP] = CA._get_double(patch_loc_cpp[iP]+p)
    for p in par_ord:
        patch[p] = np.ndarray(nPatch)
        patch[p][:]=0
        for iP in range(len(patchnames)):
            patch[p][iP] = CA._get_double(patch_loc_cpp[iP]+'ORD.'+p)

    for p in parvec:
        mat = []
        for iP in range(len(patchnames)):
            mat.append(CA._get_double_vector(patch_loc_cpp[iP]+'v'+p))
        patch[p] = np.array(mat).transpose()
        
    for p in parvec_ord:
        mat = []
        for iP in range(len(patchnames)):
            dat = CA._get_double_vector(patch_loc_cpp[iP]+'ORD.v'+p, warn=False)
            if np.any(np.isnan(dat)):
                dat[:] = CA._get_double(patch_loc_cpp[iP]+'ORD.'+p)
            mat.append(dat)
        patch[p] = np.array(mat).transpose()

    for p in parmat:
        #nMat = int(CA.get_double(patch_loc_cpp[0]+p[1]))
        nMat = 50
        patch[p[0]] = np.zeros((len(P['t']),nPatch,nMat))
        for iP in range(len(patchnames)):
            for iMat in range(nMat):
                # TODO: export full matrix, now only last idx is exported
                patch[p[0]][:,iP, iMat] = CA._get_double(patch_loc_cpp[iP]+p[0]+':'+str(iMat+1))
        
    return patch

