import numpy as np

def set_model_state(CA, Pdict):
    _set_general(CA, Pdict)
    _set_ArtVen(CA, Pdict)
    # Chamber: nothing
    _set_TriSeg(CA, Pdict)
    _set_cavity_volumes(CA, Pdict)
    _set_wall(CA, Pdict)
    _set_patch(CA, Pdict)
    # Node: nothing
    _set_bag(CA, Pdict)
    _set_valve(CA, Pdict)
    # Adapt: TODO: implement adapt

def _set_general(CA, Pdict):
    CA._set_double('Model.PFC.q0', Pdict['General']['q0'])
    CA._set_double('Model.PFC.p0', Pdict['General']['p0'])
    CA._set_double('Model.tCycle', Pdict['General']['tCycle'])
    CA._set_double('Model.PFC.FacpControl', Pdict['General']['FacpControl'])
    CA._set_double('Solver.dT', Pdict['General']['Dt'])
    CA._set_double('Model.Timings.TauAv', Pdict['General']['TauAv'])
    CA._set_double('Model.Timings.dTauAv', Pdict['General']['dTauAv'])

def _set_ArtVen(CA, Pdict):
    par = {'k','Len','A0','p0','AWall'}
    for p in par:
        CA._set_double('Model.SyArt.'+p,Pdict['ArtVen'][p][0,0])
        CA._set_double('Model.SyVen.'+p,Pdict['ArtVen'][p][1,0])
        CA._set_double('Model.PuArt.'+p,Pdict['ArtVen'][p][0,1])
        CA._set_double('Model.PuVen.'+p,Pdict['ArtVen'][p][1,1])

    par = {'p0AV','q0AV','kAV'}
    for p in par:
        CA._set_double('Model.CiSy.'+p,Pdict['ArtVen'][p][0])
        CA._set_double('Model.CiPu.'+p,Pdict['ArtVen'][p][1])

def _set_TriSeg(CA, Pdict):
    CA._set_double('Model.Peri.TriSeg.tau', Pdict['TriSeg']['Tau'])
    CA._set_double('Model.Peri.TriSeg.V', Pdict['TriSeg']['V'][-1])
    CA._set_double('Model.Peri.TriSeg.Y', Pdict['TriSeg']['Y'][-1])

def _set_cavity_volumes(CA, P):
    CA._set_double('Model.SyArt.V',P['Cavity']['V'][-1][0])
    CA._set_double('Model.SyVen.V',P['Cavity']['V'][-1][1])
    CA._set_double('Model.PuArt.V',P['Cavity']['V'][-1][2])
    CA._set_double('Model.PuVen.V',P['Cavity']['V'][-1][3])
    CA._set_double('Model.Peri.La.V',P['Cavity']['V'][-1][4])
    CA._set_double('Model.Peri.Ra.V',P['Cavity']['V'][-1][5])
    CA._set_double('Model.Peri.TriSeg.cLv.V',P['Cavity']['V'][-1][6])
    CA._set_double('Model.Peri.TriSeg.cRv.V',P['Cavity']['V'][-1][7])

def _set_wall(CA, P):
    # TODO: multipatch: Split walls into n patches

    CA._set_double('Model.Peri.La.wLa.AmDead',P['Wall']['AmDead'][P['Wall']['Name']=='La'][0])
    CA._set_double('Model.Peri.Ra.wRa.AmDead',P['Wall']['AmDead'][P['Wall']['Name']=='Ra'][0])
    CA._set_double('Model.Peri.TriSeg.wLv.AmDead',P['Wall']['AmDead'][P['Wall']['Name']=='Lv'][0])
    CA._set_double('Model.Peri.TriSeg.wSv.AmDead',P['Wall']['AmDead'][P['Wall']['Name']=='Sv'][0])
    CA._set_double('Model.Peri.TriSeg.wRv.AmDead',P['Wall']['AmDead'][P['Wall']['Name']=='Rv'][0])

patch_parameters = {'dT','LsRef','Ls0Pas','dLsPas','SfPas','k1','Lsi0Act','len_se','SfAct','vMax','TimeAct','TR','TD','CRest','VWall','AmRef'}
patch_state_variables = { 'Lsi', 'C' }

def _set_patch(CA, P):
    for iL in range(len(P['Patch']['Name'])):
        # get location name
        if P['Patch']['Name'][iL][1]=='a':
            # atria
            wall = P['Patch']['Name'][iL][0:2]
            patch = P['Patch']['Name'][iL]
            loc_name = 'Model.Peri.'+wall+'.w'+wall+'.p'+patch+'.'
        elif P['Patch']['Name'][iL][1]=='v':
            # ventricles
            wall = P['Patch']['Name'][iL][0:2]
            patch = P['Patch']['Name'][iL]
            loc_name = 'Model.Peri.TriSeg.w'+wall+'.p'+patch+'.'
        else:
            raise ValueError('Patch name not recognized')

        # set parameters
        _set_component_parameters(
            CA, patch_parameters, P['Patch'], loc_name, iL)

        # set state variables
        _set_component_parameters(
            CA, patch_state_variables, P['Patch'], loc_name, iL)


def _set_component_parameters(CA, parameter_list, sub_P, loc_name, i_seg):
    """
    Set all parameters of a component.

    loc_name ends with . (dot)
    """
    for p in parameter_list:
        if p in sub_P.keys():
            v = sub_P[p]

            # if vector pick only last
            v_shape = v.shape
            if len(v_shape)==1:
                v = v[i_seg]
            elif len(v_shape)==2:
                v = v[-1, i_seg]

            # set value
            success = CA._set_double(loc_name+p,v)
            if not success:
                print('False: ', loc_name+p)
        else:
            print('key ' + p + ' not found')

def _set_bag(CA, P):
    CA._set_double('Model.Peri.k',P['Bag']['k'])
    CA._set_double('Model.Peri.V_ref',P['Bag']['VRef'])
    CA._set_double('Model.Peri.p_ref',P['Bag']['pAdapt'])

def _set_valve(CA, P):
    par = {'AOpen','ALeak','Len'}
    par_state = {'q'}
    for iL in range(6): # TODO: implement shunts
        loc_name = 'Model.Peri.'+P['Valve']['Name'][iL]+'.'

        for p in par:
            if p in P['Valve'].keys():
                v = P['Valve'][p][iL]
                CA._set_double(loc_name+p,v)

        for p in par_state:
            if p in P['Valve'].keys():
                v = P['Valve'][p][-1][iL]
                CA._set_double(loc_name+p,v)

####################################
# Get Pdict
####################################
def get_model_state(CA):
    Pdict = {}

    Pdict['t'] = CA._get_double_vector('Solver.Time')
    Pdict['General'] = _get_general(CA)
    Pdict['Log'] = _get_log(CA)
    Pdict['ArtVen'] = _get_artven(CA)
    Pdict['Chamber'] = _get_chamber(CA)
    Pdict['TriSeg'] = _get_triseg(CA)
    Pdict['Cavity'] =_get_cavity(CA)
    Pdict['Wall'] = _get_wall(CA)
    Pdict['Patch'] = _get_patch(CA, Pdict)
    Pdict['Node'] = _get_node(CA)
    Pdict['Bag'] = _get_bag(CA)
    Pdict['Valve'] = _get_valve(CA)
    return Pdict

def _get_log(CA):
    log = {}
        # Log
    log['Code']='cpp'
    log['Version']=CA._model.getVersion(CA._idx_model)
    log['WrapperLanguage']='Python'
    log['WrapperVersion']=CA.version
    return log

def _get_general(CA):
    General = {}
    General['q0'] = CA._get_double('Model.PFC.q0')
    General['p0'] = CA._get_double('Model.PFC.p0')
    General['tCycle'] = CA._get_double('Model.tCycle')
    General['FacpControl'] = CA._get_double('Model.PFC.fac_pfc')
    General['Dt'] = CA._get_double('Solver.dT')
    General['TauAv'] = CA._get_double('Model.Timings.TauAv')
    General['dTauAv'] = CA._get_double('Model.Timings.dTauAv')

    return General

def _get_artven(CA):
    ArtVen = {}

    artvenname= np.zeros((2,), dtype=object)
    artvenname[0] = 'Sy'
    artvenname[1] = 'Pu'
    ArtVen['Name'] = artvenname
    ArtVen['n'] = 2
    ArtVen['iCavity'] = [1, 3]
    ArtVen['iWall'] = [1, 3]
    ArtVen['Adapt'] = {}

    par = {'k','l','A0','p0','A_wall'}
    for p in par:
        ArtVen[p] = np.ndarray((2,2))
        ArtVen[p][0,0] = CA._get_double('Model.SyArt.'+p)
        ArtVen[p][1,0] = CA._get_double('Model.SyVen.'+p)
        ArtVen[p][0,1] = CA._get_double('Model.PuArt.'+p)
        ArtVen[p][1,1] = CA._get_double('Model.PuVen.'+p)
    par = {'p0','q0','k'};
    for p in par:
        ArtVen[p] = np.ndarray(2)
        ArtVen[p][0] = CA._get_double('Model.CiSy.'+p)
        ArtVen[p][1] = CA._get_double('Model.CiPu.'+p)

    ArtVen['q'] = np.concatenate((CA._get_double_vector('Model.CiSy.q').reshape(-1,1),
                                  CA._get_double_vector('Model.CiPu.q').reshape(-1,1)),
                                 axis=1)
    return ArtVen

def _get_chamber(CA):
    chamber = {}
    chambername= np.zeros((2,), dtype=object)
    chambername[0] = 'La'
    chambername[1] = 'Ra'
    chamber['Name'] = chambername
    chamber['n'] = 2
    chamber['iCavity'] = [5, 6]
    chamber['iWall'] = [5, 6]
    return chamber

def _get_triseg(CA):
    triseg = {}
    trisegname= np.zeros((1,), dtype=object)
    trisegname[0] = 'v'
    triseg['Name'] = trisegname;
    triseg['n'] = 1;
    triseg['iCavity'] = 7;
    triseg['iWall'] = 7;

    par = {'V','Y','V0','Y0','VDot','YDot'};
    for p in par:
        triseg[p] = CA._get_double_vector('Model.Peri.TriSeg.'+p).reshape(-1,1)
    triseg['Tau'] = CA._get_double('Model.Peri.TriSeg.tau')
    return triseg

def _get_cavity(CA):
    cavity = {}
    cavityname= np.zeros((8,), dtype=object)
    cavityname[0] = 'SyArt'
    cavityname[1] = 'SyVen'
    cavityname[2] = 'PuArt'
    cavityname[3] = 'PuVen'
    cavityname[4] = 'La'
    cavityname[5] = 'Ra'
    cavityname[6] = 'Lv'
    cavityname[7] = 'Rv'
    cavity['Name'] = cavityname
    cavity['n'] = 8
    cavity['iNode'] = np.array([1, 2, 3, 4, 5, 6, 7, 8])
    cavity['Adapt'] = {}
    par = ['V','A','Y','p','VDot']
    cpppar = ['V','A','Y','p_trans','VDot']
    Loc = ['Model.SyArt.',
           'Model.SyVen.',
           'Model.PuArt.',
           'Model.PuVen.',
           'Model.Peri.La.',
           'Model.Peri.Ra.',
           'Model.Peri.TriSeg.cLv.',
           'Model.Peri.TriSeg.cRv.',
           ]
    for iP in range(len(par)):
        mat = []
        for iL in range(len(Loc)):
            mat.append(CA._get_double_vector(Loc[iL]+cpppar[iP]))
        cavity[par[iP]] = np.array(mat).transpose()

    return cavity

def _get_wall(CA):
    wall = {}

    wallname= np.zeros((10,), dtype=object)
    wallname[0] = 'SyArt'
    wallname[1] = 'SyVen'
    wallname[2] = 'PuArt'
    wallname[3] = 'PuVen'
    wallname[4] = 'La'
    wallname[5] = 'Ra'
    wallname[6] = 'Lv'
    wallname[7] = 'Sv'
    wallname[8] = 'Rv'
    wallname[9] = 'Peri'
    wall['Name'] = wallname
    wall['n'] = 10;
    wall['nPatch'] = np.array([0, 0, 0, 0, 1, 1, 1, 1, 1, 0])
    wall['iPatch'] = np.array([1, 1, 1, 1, 1, 2, 3, 4, 5, 6])
    # Ventricular Patch to Wall
    par = ['AmDead','VWall']
    Loc1 = ['La','Ra','Lv','Sv','Rv']
    LocCPP = ['Model.Peri.La.wLa.',
              'Model.Peri.Ra.wRa.',
              'Model.Peri.TriSeg.wLv.',
              'Model.Peri.TriSeg.wSv.',
              'Model.Peri.TriSeg.wRv.',
              ]

    for iP in range(len(par)):
        wall[par[iP]] = np.ndarray(10)
        wall[par[iP]][:]=0
        for iL in range(len(Loc1)):
            wall[par[iP]][iL+4] = CA._get_double(LocCPP[iL]+par[iP])

    par = ['Am0','DADT','T','Cm','Am','Vm','p_trans'];
    for iP in range(len(par)):
        mat = []
        for iL in range(len(Loc1)):
            mat.append(CA._get_double_vector(LocCPP[iL]+par[iP]))
        wall[par[iP]] = np.array(mat).transpose()

    # get patch data
    #allWalls=['La','Ra','Lv','Sv','Rv']
    #nPatch = 0
    #for iWall in range(len(allWalls)):
    #    wall['iPatch'][iWall+4] = int(wall['iPatch'][iWall+3] + nPatch)
    #    nPatch = self.getScalar(allWalls[iWall],'Wall','','nPatch')
    #    wall['nPatch'][iWall+4] = int(nPatch)

    return wall

def _get_patch(CA, P):
    patch = {}

    wallname = P['Wall']['Name']
    nPatch = int(sum(P['Wall']['nPatch']))
    patchnames = np.zeros((nPatch,), dtype=object)
    iPatchName = 0
    for iWall in range(len(wallname)):
        for iPatch in range(P['Wall']['nPatch'][iWall]):
            patchnames[P['Wall']['iPatch'][iWall]-1+iPatch]=wallname[iWall] + str(iPatch+1)

    patch_loc_cpp = np.array( patchnames)
    for i in range(len(patchnames)):
        patch_loc_cpp[i] = 'Model.Peri.'
        if patchnames[i][1] == 'a':
            patch_loc_cpp[i] = patch_loc_cpp[i] + patchnames[i][:2] + '.w' + patchnames[i][:2] + '.p'+patchnames[i]+'.'
        elif patchnames[i][1] == 'v':
            patch_loc_cpp[i] = patch_loc_cpp[i] + 'TriSeg.w' + patchnames[i][:2] + '.p'+patchnames[i]+'.'

    patch['Name'] = patchnames
    patch['n'] = nPatch

    par = {'dT','LsRef','Ls0Pas','dLsPas','SfPas','Lsi0Act','k1','len_se','SfAct','vMax','TimeAct','TR','TD','CRest','VWall','AmRef'}

    parvec = ['Lsi','C','T','Ef','Ls','CDot','SfEcm','SfPasT','LsiDot','Sf','DSfDEf','DADT','Am0','Am'];

    for p in par:
        patch[p] = np.ndarray(nPatch)
        patch[p][:]=0
        for iP in range(len(patchnames)):
            patch[p][iP] = CA._get_double(patch_loc_cpp[iP]+p)

    for p in parvec:
        mat = []
        for iP in range(len(patchnames)):
            mat.append(CA._get_double_vector(patch_loc_cpp[iP]+p))
        patch[p] = np.array(mat).transpose()

    return patch

def _get_node(CA):
    node = {}
    cavityname= np.zeros((8,), dtype=object)
    cavityname[0] = 'SyArt'
    cavityname[1] = 'SyVen'
    cavityname[2] = 'PuArt'
    cavityname[3] = 'PuVen'
    cavityname[4] = 'La'
    cavityname[5] = 'Ra'
    cavityname[6] = 'Lv'
    cavityname[7] = 'Rv'
    node['Name'] = cavityname
    node['n'] = 8
    node['iCavity'] = np.array([1, 2, 3, 4, 5, 6, 7, 8])
    par = ['q','p'] # Y
    cpppar = ['q','p'] # Y
    Loc = ['Model.SyArt.',
           'Model.SyVen.',
           'Model.PuArt.',
           'Model.PuVen.',
           'Model.Peri.La.',
           'Model.Peri.Ra.',
           'Model.Peri.TriSeg.cLv.',
           'Model.Peri.TriSeg.cRv.',
           ]
    for iP in range(len(par)):
        mat = []
        for iL in range(len(Loc)):
            mat.append(CA._get_double_vector(Loc[iL]+cpppar[iP]))
        node[par[iP]] = np.array(mat).transpose()

    return node

def _get_bag(CA):
    bag = {}
    bagnames= np.zeros((1,), dtype=object)
    bagnames[0] = 'Peri'
    bag['Name'] = bagnames
    bag['n'] = 1
    bag['iWall'] = [10]
    bag['pAdapt'] = CA._get_double('Model.Peri.p_ref')
    bag['p'] = CA._get_double_vector('Model.Peri.p')
    bag['k'] = CA._get_double('Model.Peri.k')
    bag['VRef'] = CA._get_double('Model.Peri.V_ref')

    return bag

def _get_valve(CA):
    valves = {}

    valvenames= np.zeros((6,), dtype=object)
    valvenames[0] = 'SyVenRa'
    valvenames[1] = 'RaRv'
    valvenames[2] = 'RvPuArt'
    valvenames[3] = 'PuVenLa'
    valvenames[4] = 'LaLv'
    valvenames[5] = 'LvSyArt'
    #valvenames[6] = 'LaRa'
    #valvenames[7] = 'LvRv'
    #valvenames[8] = 'SyArtPuArt'
    valves['Name'] = valvenames;
    valves['n'] = 9;
    valves['iNodeProx'] = [2, 6, 8, 4, 5, 7, 5, 7, 1];
    valves['iNodeDist'] = [6, 8, 3, 5, 7, 1, 6, 8, 3];

    par = ['A_open','A_leak','l'];
    for iP in range(len(par)):
        valves[par[iP]] = np.ndarray(len(valvenames))
        for iL in range(len(valvenames)):
            valves[par[iP]][iL] = CA._get_double('Model.Peri.'\
                                                +valvenames[iL]+'.'+par[iP])
    par = ['q','L','A','qDot'];
    parcpp = ['q','L','A','qDot'];
    for iP in range(len(par)):
        mat = []
        for iL in range(len(valvenames)):
            mat.append(CA._get_double_vector('Model.Peri.'\
                                            +valvenames[iL]+'.'+parcpp[iP]))
        valves[par[iP]] = np.array(mat).transpose()
    return valves
